use {
	gtk4 :: {
		Image , Label , ListBox , ListBoxRow , Popover ,
		Align :: * ,
		Orientation :: * ,
		prelude :: * ,
	} ,
	libadwaita :: * ,
} ;

use crate :: ui :: { collatable_container :: * , utility :: * } ;



pub fn create_film_collection_menu <F> ( on_sort : F ) -> gtk4 :: Box
where F : Fn (FilmsSortedBy) + 'static {
	let container = gtk4 :: Box :: builder ()
		. orientation (Horizontal)
		. halign (Center)
		. spacing (20)
		. css_classes ( ["toolbar"] )
		. build () ;

	container . append ( & create_sort_button ( & create_films_sort_menu (on_sort) ) ) ;
	container . append ( & SplitButton :: builder () . label ("Filter") . build () ) ;

	container
}

pub fn create_series_collection_menu ( on_sort : fn (SeriesSortedBy) ) -> gtk4 :: Box {
	let container = gtk4 :: Box :: builder ()
		. orientation (Horizontal)
		. halign (Center)
		. spacing (20)
		. css_classes ( ["toolbar"] )
		. build () ;

	container . append ( & create_sort_button ( & create_series_sort_menu (on_sort) ) ) ;
	container . append ( & SplitButton :: builder () . label ("Filter") . build () ) ;

	container
}

fn create_sort_button ( sort_menu : & Popover ) -> SplitButton {
	SplitButton :: builder ()
		. child ( & create_label ("Sort") )
		. popover (sort_menu)
		. build ()
}

fn create_films_sort_menu <F> ( on_sort : F ) -> Popover
where F : Fn (FilmsSortedBy) + 'static {
	let container = ListBox :: new () ;

	container . append ( & create_sort_menu_entry ( "Name" , false , true ) ) ;
	container . append ( & create_sort_menu_entry ( "Release date" , false , false ) ) ;
	container . append ( & create_sort_menu_entry ( "Runtime" , false , false ) ) ;

	container . select_row ( container . row_at_index (0) . as_ref () ) ;

	container . connect_row_activated ( move | _ , row | on_sort ( match row . index () {
		0 => FilmsSortedBy :: Name ,
		1 => FilmsSortedBy :: ReleaseDate ,
		2 => FilmsSortedBy :: Runtime ,
		_ => panic ! () ,
	} ) ) ;

	Popover :: builder ()
		. child ( & container )
		. css_classes ( ["menu"] )
		. build ()
}

fn create_series_sort_menu ( on_sort : fn (SeriesSortedBy) ) -> Popover {
	let container = ListBox :: new () ;

	container . append ( & create_sort_menu_entry ( "Name" , false , true ) ) ;
	container . append ( & create_sort_menu_entry ( "First release date" , false , false ) ) ;

	container . select_row ( container . row_at_index (0) . as_ref () ) ;

	container . connect_row_activated ( move | _ , row | on_sort ( match row . index () {
		0 => SeriesSortedBy :: Name ,
		1 => SeriesSortedBy :: FirstReleaseDate ,
		_ => panic ! () ,
	} ) ) ;

	Popover :: builder ()
		. child ( & container )
		. css_classes ( ["menu"] )
		. build ()
}

fn create_sort_menu_entry ( label : & str , reverse : bool , selected : bool ) -> ListBoxRow {
	let icon = match reverse {
		false => Image :: from_icon_name ("view-sort-ascending-symbolic") ,
		true => Image :: from_icon_name ("view-sort-descending-symbolic") ,
	} ;
	if ! selected { icon . set_opacity (0.0) }

	let container = create_horizontal_box ! (
		& Label :: builder ()
			. label (label)
			. hexpand (true)
			. halign (Start)
			. build () ,
		& icon ,
	) ;
	container . set_spacing (20) ;

	// TODO : Highlight `:selected` row using CSS, parent with `.menu` prevents default behaviour

	ListBoxRow :: builder () . child ( & container ) . build ()
}
